// Delphi console example for OSC Library
// Project/Options/Linker/EXE and DLL Options/Generate Console Application should be checked.

program osc_example_console;

uses osc_timer, osc_MIDIInput, osc_MIDIOutput, osc_SFManager, osc_Defines,
     SysUtils;

{$R *.RES}

var
   input:TOSC_MIDIInput;
   output:TOSC_MIDIOutput;
   sfman:TOSC_SFManager;

{procedure TForm1.FormCreate(Sender: TObject);
var sl:StringArray;
    i:longint;
begin
     active := false;

     timer := TOSC_Timer.Create();
     timer.setTimerCallback( TimerCallback );
     timer.SetTimerData( Handle );
     timer.SetTimerDelay( 500 );

     input := TOSC_MIDIInput.Create();
     SetLength( sl, 0 );
     input.GetListOfDevices( sl );
     i := 0;
     while ( i <> Length( sl ) ) do
     begin
          ComboBox1.Items.Add( sl[i] );
          ComboBox1.ItemIndex := 0;
          inc(i);
     end;

     output := TOSC_MIDIOutput.Create();
     SetLength( sl, 0 );
     output.GetListOfDevices( sl );
     i := 0;
     while ( i <> Length( sl ) ) do
     begin
          ComboBox2.Items.Add( sl[i] );
          ComboBox2.ItemIndex := 0;
          inc(i);
     end;

     sfman := TOSC_SFManager.Create();
     SetLength( sl, 0 );
     sfman.GetListOfDevices( sl );
     i := Length( sl );
     while ( i <> 0 ) do
     begin
          dec(i);
          ComboBox3.Items.Add( sl[i] );
          ComboBox3.ItemIndex := 0;
     end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     Stop();

     sfman.Free();

     output.Free();

     input.Free();

     timer.Free();
end;

procedure TForm1.Start();
var s:ansistring;
    i,k:longint;
begin
     if ( active ) then exit;

     if ( ComboBox1.ItemIndex <> -1 ) then
     begin
          s := ComboBox1.Items[ ComboBox1.ItemIndex ];

     end;

     if ( ComboBox2.ItemIndex <> -1 ) then
     begin
          s := ComboBox2.Items[ ComboBox2.ItemIndex ];
          output.GetDevID( s, k );
          output.Open( k );
     end;

     if ( ComboBox3.ItemIndex <> -1 ) then
     begin
          s := ComboBox3.Items[ ComboBox3.ItemIndex ];
          sfman.GetDevNo( s, k );
          sfman.Open( k );
          ListBox1.Items.Clear();
          for k := 0 to 127 do for i := 0 to 127 do
           if ( sfman.GetLoadedPresetDescriptor( k, i, s ) = OSC_ERR_OK ) then
          begin
               ListBox1.Items.Add( inttostr( k ) + ':' + inttostr( i ) + '   ' + s );
               ListBox1.ItemIndex := 0;
          end;
     end;

     output.Connect( input.GetHandle() );

     timer.StartTimer();

     active := true;
end;

procedure TForm1.Stop();
begin
     if ( not active ) then exit;

     timer.StopTimer();
     Label1.Color := clGreen;

     output.Disconnect( input.GetHandle() );

     sfman.Close();

     output.Close();

     input.Stop();
     input.Close();

     active := false;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
     Start();
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
     Stop();
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
     if ( active ) then
     begin
          Stop();
          Start();
     end;
end;

procedure TForm1.ListBox1Change(Sender: TObject);
var s:ansistring;
begin
     if ( active ) then if ( ListBox1.ItemIndex <> -1 ) then
     begin
          s := ListBox1.Items[ ListBox1.ItemIndex ];
          output.SendMIDIMessage(
           MIDI_MESSAGE_CONTROL_CHANGE,
           MIDI_MESSAGE_CONTROL_BANK_SELECT_COARSE,
           strtointdef( copy( s, 1, pos( ':', s ) - 1 ), 0 )
          );
          output.SendMIDIMessage(
           MIDI_MESSAGE_PROGRAM_CHANGE,
           strtointdef( copy( s, pos( ':', s ) + 1, pos( ' ', s ) - pos( ':', s ) - 1 ), 0 ),
           0
          );
     end;
end;}

var sl:StringArray;
    s:ansistring;
    i,k:longint;
    bTemp:boolean;
begin
     input := TOSC_MIDIInput.Create();
     SetLength( sl, 0 );
     input.GetListOfDevices( sl );
     writeln;
     writeln( 'Choose MIDI Input:' );
     writeln;
     i := 0;
     while ( i <> Length( sl ) ) do
     begin
          writeln( inttostr( i ) + ': ' + sl[i] );
          inc(i);
     end;
     writeln;
     write( '> ' );
     readln( i );
     writeln;
     input.GetDevID( sl[i], i );
     input.Open( i );
     input.Start();

     output := TOSC_MIDIOutput.Create();
     SetLength( sl, 0 );
     output.GetListOfDevices( sl );
     writeln( 'Choose MIDI Output:' );
     writeln;
     i := 0;
     while ( i <> Length( sl ) ) do
     begin
          writeln( inttostr( i ) + ': ' + sl[i] );
          inc(i);
     end;
     writeln;
     write( '> ' );
     readln( i );
     writeln;
     output.GetDevID( sl[i], i );
     output.Open( i );

     output.Connect( input.GetHandle() );

     sfman := TOSC_SFManager.Create();
     SetLength( sl, 0 );
     sfman.GetListOfDevices( sl );
     writeln( 'Choose SoundFont Device:' );
     writeln;
     i := Length( sl );
     while ( i <> 0 ) do
     begin
          dec(i);
          writeln( inttostr( Length( sl ) - i - 1 ) + ': ' + sl[i] );
     end;
     writeln;
     write( '> ' );
     readln( i );
     writeln;
     sfman.GetDevNo( sl[i], i );
     sfman.Open( i );
     bTemp := false;
     for i := 0 to 127 do if ( sfman.IsMIDIBankUsed( i ) <> -1 ) then
     begin
          bTemp := true;
          break;
     end;
     if ( bTemp ) then
     begin
          writeln( 'Choose bank:' );
          for i := 0 to 127 do if ( sfman.IsMIDIBankUsed( i ) <> -1 ) then
           if ( sfman.GetLoadedBankDescriptor( i, s ) = OSC_ERR_OK ) then
           writeln( inttostr( i ) + ': ' + s );
          writeln;
          write( '> ' );
          readln( k );
          writeln;
          bTemp := false;
          for i := 0 to 127 do
           if ( sfman.GetLoadedPresetDescriptor( k, i, s ) = OSC_ERR_OK ) then
          begin
               bTemp := true;
               break;
          end;
          if ( bTemp ) then
          begin
               writeln( 'Choose preset:' );
               for i := 0 to 127 do
                if ( sfman.GetLoadedPresetDescriptor( k, i, s ) = OSC_ERR_OK ) then
                writeln( inttostr( i ) + ': ' + s );
               writeln;
               write( '> ' );
               readln( i );
               writeln;
               output.SendMIDIMessage(
                MIDI_MESSAGE_CONTROL_CHANGE,
                MIDI_MESSAGE_CONTROL_BANK_SELECT_COARSE,
                k
               );
               output.SendMIDIMessage(
                MIDI_MESSAGE_PROGRAM_CHANGE,
                i,
                0
               );
          end
          else
          begin
               writeln( 'no presets in bank' );
               writeln;
          end;
     end
     else
     begin
          writeln( 'no banks loaded' );
          writeln;
     end;

     writeln( 'MIDI In connected to MIDI Out. Press ENTER to disconnect and exit...' );
     readln;

     output.Disconnect( input.GetHandle() );

     sfman.Close();

     output.Close();

     input.Stop();
     input.Close();
end.
