// Delphi GUI example for OSC Library

unit mainform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, osc_timer, osc_MIDIInput, osc_MIDIOutput, osc_SFManager, osc_Defines;

type
  TForm1 = class(TForm)
    ComboBox1: TComboBox;
    ComboBox2: TComboBox;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    ComboBox3: TComboBox;
    ListBox1: TListBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure ListBox1Change(Sender: TObject);
  private
    { Private declarations }
    procedure Blink( var m:Msg ); message WM_USER + 1;
  public
    { Public declarations }
    timer:TOSC_Timer;
    input:TOSC_MIDIInput;
    output:TOSC_MIDIOutput;
    sfman:TOSC_SFManager;
    active:boolean;
    procedure Start();
    procedure Stop();
    procedure TimerCallback( data:Cardinal );
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Blink( var m:Msg );
begin
     if ( Label1.Color = clGreen ) then Label1.Color := clLime else Label1.Color := clGreen;
end;

procedure TForm1.TimerCallback( data:Cardinal );
begin
     repeat until PostMessage( data, WM_USER + 1, 0, 0 );
     // We can't do something in VCL thread, i.e. with main window directly
     // since on this point we are in _another_ _thread_.
end;

procedure TForm1.FormCreate(Sender: TObject);
var sl:StringArray;
    i:longint;
begin
     active := false;

     timer := TOSC_Timer.Create();
     timer.setTimerCallback( TimerCallback );
     timer.SetTimerData( Handle );
     timer.SetTimerDelay( 500 );

     input := TOSC_MIDIInput.Create();
     SetLength( sl, 0 );
     input.GetListOfDevices( sl );
     i := 0;
     while ( i <> Length( sl ) ) do
     begin
          ComboBox1.Items.Add( sl[i] );
          ComboBox1.ItemIndex := 0;
          inc(i);
     end;

     output := TOSC_MIDIOutput.Create();
     SetLength( sl, 0 );
     output.GetListOfDevices( sl );
     i := 0;
     while ( i <> Length( sl ) ) do
     begin
          ComboBox2.Items.Add( sl[i] );
          ComboBox2.ItemIndex := 0;
          inc(i);
     end;

     sfman := TOSC_SFManager.Create();
     SetLength( sl, 0 );
     sfman.GetListOfDevices( sl );
     i := Length( sl );
     while ( i <> 0 ) do
     begin
          dec(i);
          ComboBox3.Items.Add( sl[i] );
          ComboBox3.ItemIndex := 0;
     end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     Stop();

     sfman.Free();

     output.Free();

     input.Free();

     timer.Free();
end;

procedure TForm1.Start();
var s:ansistring;
    i,k:longint;
begin
     if ( active ) then exit;

     if ( ComboBox1.ItemIndex <> -1 ) then
     begin
          s := ComboBox1.Items[ ComboBox1.ItemIndex ];
          input.GetDevID( s, k );
          input.Open( k );
          input.Start();
     end;

     if ( ComboBox2.ItemIndex <> -1 ) then
     begin
          s := ComboBox2.Items[ ComboBox2.ItemIndex ];
          output.GetDevID( s, k );
          output.Open( k );
     end;

     if ( ComboBox3.ItemIndex <> -1 ) then
     begin
          s := ComboBox3.Items[ ComboBox3.ItemIndex ];
          sfman.GetDevNo( s, k );
          sfman.Open( k );
          ListBox1.Items.Clear();
          for k := 0 to 127 do for i := 0 to 127 do
           if ( sfman.GetLoadedPresetDescriptor( k, i, s ) = OSC_ERR_OK ) then
          begin
               ListBox1.Items.Add( inttostr( k ) + ':' + inttostr( i ) + '   ' + s );
               ListBox1.ItemIndex := 0;
          end;
     end;

     output.Connect( input.GetHandle() );

     timer.StartTimer();

     active := true;
end;

procedure TForm1.Stop();
begin
     if ( not active ) then exit;

     timer.StopTimer();
     Label1.Color := clGreen;

     output.Disconnect( input.GetHandle() );

     sfman.Close();

     output.Close();

     input.Stop();
     input.Close();

     active := false;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
     Start();
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
     Stop();
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
     if ( active ) then
     begin
          Stop();
          Start();
     end;
end;

procedure TForm1.ListBox1Change(Sender: TObject);
var s:ansistring;
begin
     if ( active ) then if ( ListBox1.ItemIndex <> -1 ) then
     begin
          s := ListBox1.Items[ ListBox1.ItemIndex ];
          output.SendMIDIMessage(
           MIDI_MESSAGE_CONTROL_CHANGE,
           MIDI_MESSAGE_CONTROL_BANK_SELECT_COARSE,
           strtointdef( copy( s, 1, pos( ':', s ) - 1 ), 0 )
          );
          output.SendMIDIMessage(
           MIDI_MESSAGE_PROGRAM_CHANGE,
           strtointdef( copy( s, pos( ':', s ) + 1, pos( ' ', s ) - pos( ':', s ) - 1 ), 0 ),
           0
          );
     end;
end;

end.
