//*********************************************//
// this file is a translation of OSC_Defines.h //
//*********************************************//

{
	This file is a part of the OSC Library (Open Sound Code).
	Visit the OSC website at http://www.pvv.org/~thammer/HammerSound/osc/
	for the complete OSC library, the latest update, documentation,
	user forum, etc.

	Allthough the source code to the OSC Library is publicly available,
	use and distribution of the OSC Library is governed by a license agreement.
	The License Agreement can be found at the OSC website.
}


unit OSC_Defines;

interface

uses
  Windows;

// MIDI data types

type
    MIDIDataShort = byte;
    MIDIDataLong = word;
    MIDITimeStamp = dword;

const
// MIDI Messages

       // Masks
       MIDI_MESSAGE_CHANNEL_MASK	      =	$0F;
       MIDI_MESSAGE_TYPE_MASK		      = $F0;

       // Message Categories (Lower bounds)
       MIDI_MESSAGE_SYSTEM_VOICE	      =	$80;
       MIDI_MESSAGE_SYSTEM_COMMON	      =	$F0;
       MIDI_MESSAGE_SYSTEM_REALTIME	      =	$F8;

       // Message Types
       MIDI_MESSAGE_NOTE_OFF		      = $80;
       MIDI_MESSAGE_NOTE_ON		      =	$90;
       MIDI_MESSAGE_KEY_PRESSURE	      =	$A0;
       MIDI_MESSAGE_CONTROL_CHANGE	      =	$B0;
       MIDI_MESSAGE_PROGRAM_CHANGE	      =	$C0;
       MIDI_MESSAGE_CHANNEL_PRESSURE	      = $D0;
       MIDI_MESSAGE_PITCH_WHEEL		      = $E0;

       MIDI_MESSAGE_SYSTEM_COMMON_SYSTEM_EXCLUSIVE	      =	$F0;
       MIDI_MESSAGE_SYSTEM_COMMON_MTC_QUARTER_FRAME	      =	$F1;
       MIDI_MESSAGE_SYSTEM_COMMON_SONG_POSITION_POINTER	      = $F2;
       MIDI_MESSAGE_SYSTEM_COMMON_SONG_SELECT		      = $F3;
       MIDI_MESSAGE_SYSTEM_COMMON_TUNE_REQUEST		      = $F6;
       MIDI_MESSAGE_SYSTEM_COMMON_END_OF_SYSEX		      = $F7;

       MIDI_MESSAGE_SYSTEM_REALTIME_MIDI_CLOCK		      = $F8;
       MIDI_MESSAGE_SYSTEM_REALTIME_MIDI_START		      = $FA;
       MIDI_MESSAGE_SYSTEM_REALTIME_MIDI_CONTINUE	      = $FB;
       MIDI_MESSAGE_SYSTEM_REALTIME_MIDI_STOP		      = $FC;
       MIDI_MESSAGE_SYSTEM_REALTIME_ACTIVE_SENSE	      = $FE;
       MIDI_MESSAGE_SYSTEM_REALTIME_RESET		      = $FF;

       // Controller Messages
       MIDI_MESSAGE_CONTROL_BANK_SELECT_COARSE	              = $00;
       MIDI_MESSAGE_CONTROL_BANK_SELECT_FINE	              = 32;
       MIDI_MESSAGE_CONTROL_MOD_WHEEL_MSB		      = $01;
       MIDI_MESSAGE_CONTROL_DATA_ENTRY_MSB		      = $06;
       MIDI_MESSAGE_CONTROL_VOLUME_MSB		              = $07;
       MIDI_MESSAGE_CONTROL_PAN			              = $0A;
       MIDI_MESSAGE_CONTROL_DATA_ENTRY_LSB		      = $26;
       MIDI_MESSAGE_CONTROL_NRPN_LSB		              = $62;
       MIDI_MESSAGE_CONTROL_NRPN_MSB		              = $63;
       MIDI_MESSAGE_CONTROL_ALL_NOTES_OFF		      = $7B;



// Data conversion
type
   DWORD_4BYTES = record
     case iSize:byte of
     4: ( d:dword );
     1:	( b0,b1,b2,b3:byte );
   end;


// String array for list of devices
   StringArray = array of ansistring;


// Error constants

type OSC_ERR = longint;

const

   OSC_ERR_BASE_SFMAN = 500;

   OSC_ERR_OK = 0;
   OSC_ERR_FAIL = 1;
   OSC_ERR_ILLEGAL_DEVICE_ID = 2;
   OSC_ERR_ILLEGAL_DEVICE_NAME = 3;
   OSC_ERR_MM = 100;

   OSC_ERR_TIMER_UNAVAILABLE = 200;

   OSC_ERR_SFMAN_COULDNT_GET_SFMAN_FUNCTION_TABLE	= OSC_ERR_BASE_SFMAN;
   OSC_ERR_SFMAN_COULDNT_GET_SFMAN_INTERFACE		= OSC_ERR_BASE_SFMAN + 1;
   OSC_ERR_SFMAN_INVALID_DEVICE_NUMBER			= OSC_ERR_BASE_SFMAN + 2;

implementation


end.
