//*************************************************************************//
// this file is a translation of OSC_MIDIMessage.h and OSC_MIDIMessage.cpp //
//*************************************************************************//

{
	This file is a part of the OSC Library (Open Sound Code).
	Visit the OSC website at http://www.pvv.org/~thammer/HammerSound/osc/
	for the complete OSC library, the latest update, documentation,
	user forum, etc.

	Allthough the source code to the OSC Library is publicly available,
	use and distribution of the OSC Library is governed by a license agreement.
	The License Agreement can be found at the OSC website.
}


unit OSC_MIDIMessage;

interface

uses
  OSC_Defines;

(**
	A helper-class to represent a MIDI message. The class has simple access functions
	to make it easy to construct a MIDI message.
*)

type
  TOSC_MIDIMessage = class
    private
      m_timeStamp: MIDITimeStamp;
      m_status: MIDIDataShort;
      m_data1: MIDIDataShort;
      m_data2: MIDIDataShort;
    public
      constructor Create( status:MIDIDataShort = 0; data1:MIDIDataShort = 0;
       data2:MIDIDataShort = 0; timeStamp:MIDITimeStamp = 0 );
      destructor Destroy();override;
      procedure Clear();
      procedure Init( status:MIDIDataShort = 0; data1:MIDIDataShort = 0;
       data2:MIDIDataShort = 0; timeStamp:MIDITimeStamp = 0 );
      function GetTimeStamp():MIDITimeStamp;
      function IsVoiceMessage():boolean;
      function IsKeyMessage():boolean;
      function IsNoteOnOff():boolean;
      function IsNoteOn():boolean;
      function IsControlChange():boolean;
      function IsBankSelect0():boolean;
      function IsBankSelect32():boolean;
      function IsReset():boolean;
      function IsProgramChange():boolean;
      function GetMessageType():MIDIDataShort;
      procedure SetMessageType( messagetype:MIDIDataShort );
      function GetChannel():MIDIDataShort;
      procedure SetChannel( channel:MIDIDataShort );
      function GetNoteNumber():MIDIDataShort;
      function GetNoteVelocity():MIDIDataShort;
      procedure SetNoteNumber( number:MIDIDataShort );
      procedure SetNoteVelocity( velocity:MIDIDataShort );
      function GetKeyPressure():MIDIDataShort;// e.g. After touch
      function GetControllerNumber():MIDIDataShort;
      procedure SetControllerNumber( data:MIDIDataShort );
      function GetControllerValue():MIDIDataShort;
      procedure SetControllerValue( data:MIDIDataShort );
      function GetProgram():MIDIDataShort;
      procedure SetProgram( data:MIDIDataShort );
      function GetChannelPressure():MIDIDataShort; // e.g. Channel Pressure
      function GetPitchWheel():MIDIDataLong;
      function GetMIDIDataLong():MIDIDataLong;
      function GetDataByte1():MIDIDataShort;
      procedure SetDataByte1( data:MIDIDataShort );
      function GetDataByte2():MIDIDataShort;
      procedure SetDataByte2( data:MIDIDataShort );
      function GetStatusByte():MIDIDataShort;
      procedure SetStatusByte( data:MIDIDataShort );
  end;

implementation

constructor TOSC_MIDIMessage.Create( status:MIDIDataShort = 0; data1:MIDIDataShort = 0;
 data2:MIDIDataShort = 0; timeStamp:MIDITimeStamp = 0 );
begin
     inherited Create();
     Init( status, data1, data2, timeStamp );
end;

///
destructor TOSC_MIDIMessage.Destroy();
begin
     inherited Destroy();
end;

///
procedure TOSC_MIDIMessage.Clear();
begin
     Init();
end;

///
procedure TOSC_MIDIMessage.Init( status:MIDIDataShort = 0; data1:MIDIDataShort = 0;
 data2:MIDIDataShort = 0; timeStamp:MIDITimeStamp = 0 );
begin
     m_status := status;
     m_data1 := data1;
     m_data2 := data2;
     m_timeStamp := timeStamp;
end;

///
function TOSC_MIDIMessage.GetTimeStamp():MIDITimeStamp;
begin
     Result := m_timeStamp;
end;

///
function TOSC_MIDIMessage.IsVoiceMessage():boolean;
begin
     Result := m_status < MIDI_MESSAGE_SYSTEM_COMMON;
end;

///
function TOSC_MIDIMessage.IsKeyMessage():boolean;
begin
     Result := m_status < MIDI_MESSAGE_CONTROL_CHANGE;
end;

///
function TOSC_MIDIMessage.IsNoteOnOff():boolean;
begin
     Result := m_status < MIDI_MESSAGE_KEY_PRESSURE;
end;

function TOSC_MIDIMessage.IsNoteOn():boolean;
begin
     Result := ( ( m_status < MIDI_MESSAGE_KEY_PRESSURE )
      and ( m_status >= MIDI_MESSAGE_NOTE_ON ) and ( m_data2 <> 0 ) );
end;

function TOSC_MIDIMessage.IsControlChange():boolean;
begin
     Result := ( ( m_status >= MIDI_MESSAGE_CONTROL_CHANGE )
      and ( m_status < MIDI_MESSAGE_PROGRAM_CHANGE ) );
end;

function TOSC_MIDIMessage.IsBankSelect0():boolean;
begin
     Result := ( m_data1 = MIDI_MESSAGE_CONTROL_BANK_SELECT_COARSE );
end;

function TOSC_MIDIMessage.IsBankSelect32():boolean;
begin
     Result := ( m_data1 = MIDI_MESSAGE_CONTROL_BANK_SELECT_FINE );
end;

function TOSC_MIDIMessage.IsReset():boolean;
begin
     Result := ( m_status = MIDI_MESSAGE_SYSTEM_REALTIME_RESET );
end;

function TOSC_MIDIMessage.IsProgramChange():boolean;
begin
     Result := ( ( m_status >= MIDI_MESSAGE_PROGRAM_CHANGE )
      and ( m_status < MIDI_MESSAGE_PROGRAM_CHANGE + 16 ) );
end;

///
function TOSC_MIDIMessage.GetMessageType():MIDIDataShort;
begin
     if ( m_status >= MIDI_MESSAGE_SYSTEM_COMMON ) then
     begin	// system common or system realtime
          Result := m_status;
     end
     else
     begin	// system voice
          if ( ( ( m_status and MIDI_MESSAGE_TYPE_MASK ) = MIDI_MESSAGE_NOTE_ON )
           and ( m_data2 = 0 ) ) then
           Result := MIDI_MESSAGE_NOTE_OFF
          else
           Result := m_status and MIDI_MESSAGE_TYPE_MASK;
     end;
end;

///
procedure TOSC_MIDIMessage.SetMessageType( messagetype:MIDIDataShort );
begin
     m_status := ( m_status and MIDI_MESSAGE_CHANNEL_MASK ) or
      ( messagetype and MIDI_MESSAGE_TYPE_MASK );
end;

///
function TOSC_MIDIMessage.GetChannel():MIDIDataShort;
begin
     Result := ( m_status and MIDI_MESSAGE_CHANNEL_MASK );
end;

///
procedure TOSC_MIDIMessage.SetChannel( channel:MIDIDataShort );
begin
     m_status := ( m_status and MIDI_MESSAGE_TYPE_MASK ) // remove old channel info
      or ( channel and MIDI_MESSAGE_CHANNEL_MASK ); // add new channel info
end;

///
function TOSC_MIDIMessage.GetNoteNumber():MIDIDataShort;
begin
     Result := m_data1;
end;

///
function TOSC_MIDIMessage.GetNoteVelocity():MIDIDataShort;
begin
     Result := m_data2;
end;

///
procedure TOSC_MIDIMessage.SetNoteNumber( number:MIDIDataShort );
begin
     m_data1 := number;
end;

///
procedure TOSC_MIDIMessage.SetNoteVelocity( velocity:MIDIDataShort );
begin
     m_data2 := velocity;
end;

///
function TOSC_MIDIMessage.GetKeyPressure():MIDIDataShort;// e.g. After touch
begin
     Result := m_data2;
end;

///
function TOSC_MIDIMessage.GetControllerNumber():MIDIDataShort;
begin
     Result := m_data1;
end;

///
procedure TOSC_MIDIMessage.SetControllerNumber( data:MIDIDataShort );
begin
     m_data1 := data;
end;

///
function TOSC_MIDIMessage.GetControllerValue():MIDIDataShort;
begin
     Result := m_data2;
end;

///
procedure TOSC_MIDIMessage.SetControllerValue( data:MIDIDataShort );
begin
     m_data2 := data;
end;

///
function TOSC_MIDIMessage.GetProgram():MIDIDataShort;
begin
     Result := m_data1;
end;

///
procedure TOSC_MIDIMessage.SetProgram( data:MIDIDataShort );
begin
     m_data1 := data;
end;

///
function TOSC_MIDIMessage.GetChannelPressure():MIDIDataShort; // e.g. Channel Pressure
begin
     Result := m_data1;
end;

///
function TOSC_MIDIMessage.GetPitchWheel():MIDIDataLong;
begin
     Result := GetMIDIDataLong();
end;

///
function TOSC_MIDIMessage.GetMIDIDataLong():MIDIDataLong;
var data14bit:word;
begin
     data14bit := word(m_data2);
     data14bit := ( data14bit shl 7 );
     data14bit := ( data14bit or word(m_data1) );
     Result := data14bit;
end;

///
function TOSC_MIDIMessage.GetDataByte1():MIDIDataShort;
begin
     Result := m_data1;
end;

///
procedure TOSC_MIDIMessage.SetDataByte1( data:MIDIDataShort );
begin
     m_data1 := data;
end;

///
function TOSC_MIDIMessage.GetDataByte2():MIDIDataShort;
begin
     Result := m_data2;
end;

///
procedure TOSC_MIDIMessage.SetDataByte2( data:MIDIDataShort );
begin
     m_data2 := data;
end;

///
function TOSC_MIDIMessage.GetStatusByte():MIDIDataShort;
begin
     Result := m_status;
end;

///
procedure TOSC_MIDIMessage.SetStatusByte( data:MIDIDataShort );
begin
     m_status := data;
end;

end.
