//****************************************************//
// this file is a translation of OSC_MultiThreading.h //
//****************************************************//


unit OSC_MultiThreading;

interface

uses
  Windows;

(* Multithreading support. Wrapper for whatever critical section we have
*)

type
  TOSC_CritSec = class
      m_CritSec:TRTLCriticalSection;
    public
          ///
      constructor Create();
          ///
      destructor Destroy();override;
          /// Lock critical section. Block execution if critical section allready lock.
      procedure Lock();
          /// Unlock critrical section. Unblock threads waiting for this critical section.
      procedure Unlock();
  end;
  pTOSC_CritSec = ^TOSC_CritSec;


(** Multithreading support. Locks a critical section, and unlocks it automatically
	when the lock goes out of scope
*)

  TOSC_AutoLock = class
    protected
      m_pLock:pTOSC_CritSec;
    public
          ///
      constructor Create( plock:pTOSC_CritSec );
          ///
      destructor Destroy();override;
  end;



implementation

constructor TOSC_CritSec.Create();
begin
     inherited Create();
     InitializeCriticalSection( m_CritSec );
end;

destructor TOSC_CritSec.Destroy();
begin
     DeleteCriticalSection( m_CritSec );
     inherited Destroy();
end;

procedure TOSC_CritSec.Lock();
begin
     EnterCriticalSection( m_CritSec );
end;

procedure TOSC_CritSec.Unlock();
begin
     LeaveCriticalSection( m_CritSec );
end;

constructor TOSC_AutoLock.Create( plock:pTOSC_CritSec );
begin
     inherited Create();
     m_pLock := plock;
     m_pLock^.Lock();
end;

destructor TOSC_AutoLock.Destroy();
begin
     m_pLock^.Unlock();
     inherited Destroy();
end;


end.
