//*******************************************//
// this file is a translation of OSC_Timer.h //
//*******************************************//


unit OSC_Timer;

interface

uses
  Windows,
  mmsystem;

(*

The multimedia timer runs IN ITS OWN THREAD.
After the event is activated, it calls the specified callback function.

Example of use:

>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
unit OSC_Timer_test;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, OSC_Timer, Dialogs;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    t:TOSC_Timer;
    t2:TOSC_Timer;
    procedure TimerCallback( data:Cardinal );
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure TForm1.TimerCallback( data:Cardinal );
begin
     ShowMessage( 'Ping' + inttostr( data ) );
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     t := TOSC_Timer.Create();
     t.setTimerCallback( TimerCallback );
     t.SetTimerData( 1 );
     t.SetTimerDelay( 2000 );
     t.StartTimer();
     t2 := TOSC_Timer.Create();
     t2.setTimerCallback( TimerCallback );
     t2.SetTimerData( 2 );
     t2.SetTimerDelay( 3000 );
     t2.StartTimer();
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     t2.StopTimer();
     t2.Free();
     t.StopTimer();
     t.Free();
end;

end.
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*)

type
  TOSC_TimerCallback = procedure ( data:Cardinal ) of object; register;
  pTOSC_TimerCallback = ^TOSC_TimerCallback;
  TOSC_TimerCallbackStruct = record
    Proc:pTOSC_TimerCallback;
    Data:Cardinal;
  end;
  pTOSC_TimerCallbackStruct = ^TOSC_TimerCallbackStruct;

  pTOSC_Timer = ^TOSC_Timer;
  TOSC_Timer = class
    private
      m_timerID               :UINT;
      m_pTimerCallback        :TOSC_TimerCallbackStruct;
      m_delay                 :UINT;
      m_resolution            :UINT;
    public
      constructor Create();
      destructor Destroy();override;
      procedure SetTimerCallback( pTimerCallback:TOSC_TimerCallback );
      procedure SetTimerData( i:Cardinal );
      procedure SetTimerDelay( delay:integer; resolution:integer = -1 );
      function StartTimer():integer;
      procedure StopTimer();
      function GetTimerID():integer;
      function TimerIsRunning():boolean;
      function GetTimerDelay():UINT;
end;


implementation

procedure TimerCallback( uTimerID, uMsg:UINT; dwInstance, dwParam1, dwParam2:DWORD );stdcall;
var r:TOSC_TimerCallbackStruct;
begin
     r := ( pTOSC_TimerCallbackStruct(dwInstance) )^;
     asm
        push  ebx
        mov   ebx, r.Data
        call  r.Proc
        pop   ebx
     end;
end;

constructor TOSC_Timer.Create();
begin
     inherited Create();
     m_timerID := 0;
     setTimerDelay( 1000 );
     m_pTimerCallback.Proc := nil;
     m_pTimerCallback.Data := 0;
end;

destructor TOSC_Timer.Destroy();
begin
     if ( m_timerID <> 0 ) then timeKillEvent( m_timerID );
     inherited Destroy();
end;

procedure TOSC_Timer.SetTimerCallback( pTimerCallback:TOSC_TimerCallback );
begin
     m_pTimerCallback.Proc := @pTimerCallback;
end;

procedure TOSC_Timer.SetTimerDelay( delay:integer; resolution:integer = -1 );
begin
     m_delay := delay;
     if ( resolution = -1 ) then
      m_resolution := m_delay div 2
     else
      m_resolution := resolution;
end;

function TOSC_Timer.StartTimer():integer;
begin
     if ( m_pTimerCallback.Proc = nil ) then
     begin
          // ??? if ( m_timerID = 0 ) then Result := 1 else
          Result := 0; // you must set the callback class reference first!
     end
     else
     begin
          m_timerID := timeSetEvent( m_delay, m_resolution, TimerCallback,
           DWORD(@m_pTimerCallback), TIME_PERIODIC );
          Result := m_timerID;
     end;
end;

procedure TOSC_Timer.StopTimer();
begin
     if ( m_timerID <> 0 ) then timeKillEvent( m_timerID );
     m_timerID := 0;
end;

function TOSC_Timer.GetTimerID():integer;
begin
     Result := m_timerID;
end;

function TOSC_Timer.TimerIsRunning():boolean;
begin
     Result := ( m_timerID <> 0 );
end;

function TOSC_Timer.GetTimerDelay():UINT;
begin
     Result := m_delay;
end;

procedure TOSC_Timer.SetTimerData( i:Cardinal );
begin
     m_pTimerCallback.Data := i;
end;

end.
